﻿--[[
    Name:           Titan Mount
	Description:	Titan Panel plugin for mounting.
	Author:         Skydragon247
	Version:        6.0.0
    --]]

local MountID = "Mount"
local MountVersion = "6.0.0"
local MountC1 = "|cffffffff"
local MountC2 = "|cff993300"
local MountC3 = "|cff00ff00"
local MountC4 = "|cffffcc00"
local MountC5 = "|cff999999"
local alphabet = {
					{letter = 'A', found = false, mountIDs = {}},
					{letter = 'B', found = false, mountIDs = {}},
					{letter = 'C', found = false, mountIDs = {}},
					{letter = 'D', found = false, mountIDs = {}},
					{letter = 'E', found = false, mountIDs = {}},
					{letter = 'F', found = false, mountIDs = {}},
					{letter = 'G', found = false, mountIDs = {}},
					{letter = 'H', found = false, mountIDs = {}},
					{letter = 'I', found = false, mountIDs = {}},
					{letter = 'J', found = false, mountIDs = {}},
					{letter = 'K', found = false, mountIDs = {}},
					{letter = 'L', found = false, mountIDs = {}},
					{letter = 'M', found = false, mountIDs = {}},
					{letter = 'N', found = false, mountIDs = {}},
					{letter = 'O', found = false, mountIDs = {}},
					{letter = 'P', found = false, mountIDs = {}},
					{letter = 'Q', found = false, mountIDs = {}},
					{letter = 'R', found = false, mountIDs = {}},
					{letter = 'S', found = false, mountIDs = {}},
					{letter = 'T', found = false, mountIDs = {}},
					{letter = 'U', found = false, mountIDs = {}},
					{letter = 'V', found = false, mountIDs = {}},
					{letter = 'W', found = false, mountIDs = {}},
					{letter = 'X', found = false, mountIDs = {}},
					{letter = 'Y', found = false, mountIDs = {}},
					{letter = 'Z', found = false, mountIDs = {}},
				 }
local TPMountSFM = true
local mount = nil
local mountID = nil
local mountNumber = nil
local favSelection1, favSelection2, favSelection3

--Event fire when the addon is first loaded onto the UI
function TitanPanelMountButton_OnLoad(self)
    DEFAULT_CHAT_FRAME:AddMessage(MountC1.."Titan Panel "..MountC2.."["..MountID.."] "..MountC1.."v"..MountC2..MountVersion.." "..MountC1.."by |cff999999Skydragon247")

    TitanPanelMountButton:RegisterEvent("ADDON_LOADED")
    TitanPanelMountButton:RegisterEvent("PLAYER_LOGOUT")
  
    self.registry = {
		id = MountID,
	    version = MountVersion,
        menuText = MountID,
        buttonTextFunction = "TitanPanelMountButton_GetButtonText",	
        category = "General",
        tooltipTitle = MountC4..MountID,
		tooltipTextFunction = "TitanPanelMountButton_GetTooltipText",
        iconButtonWidth = 16,
        iconWidth = 16,        
        savedVariables = {
			ShowIcon = 1,
			ShowLabelText = 1,  
            ShowTooltipText = 1,
            ShowMenuOptions = 1,
            ShowAdvancedMenus = 1,                
            }}
			
	DetectUsableMounts()
	
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("CHAT_MSG_SYSTEM");
end


function DetectUsableMounts()
    local mountName, owned
	
	for i = 1, table.getn(alphabet) do
		--For each mount, do we have at least one mount beginning with the current letter in the alphabet (uppercase)?
		--First, check to make sure it isnt already set to true. That way we dont have to waste our time going through the entire list of mounts
		if not alphabet[i].found then
			--Lets do some searching bitches!
			for j = 1, C_MountJournal.GetNumMounts() do
				--Get dat mount info we need...
				mountName, _, _, _, owned, _, _, _, _, _, _  = C_MountJournal.GetMountInfo(j)
				
				--If the account owns the mount and it starts with the current letter we're on, set found to true for the letter
				if alphabet[i].letter == strupper(strsub(mountName, 1, 1)) and owned then
					alphabet[i].found = true
					table.insert(alphabet[i].mountIDs, j)
				end
			end
		end
	end
end

function AddNewMountID(mountNamePassed)
	for i = 1, C_MountJournal.GetNumMounts() do	
		local mountName, owned
		--Get dat mount info we need...
		mountName, _, _, _, owned, _, _, _, _, _, _  = C_MountJournal.GetMountInfo(i)
		--If the account owns the mount and it starts with the current letter we're on, set found to true for the letter
		if mountNamePassed == mountName then
			local letter = strupper(strsub(mountName, 1, 1))
			
			--Got it, but do we have it (to prevent someone changing code and adding a mount they dont have)
			for j = 1, table.getn(alphabet) do
				if alphabet[j].letter == letter then
					--It could be a new letter, mark it as found
					if not alphabet[j].found then
						alphabet[j].found = true
					end
					
					--Got the letter, now add it to that letters list of mountIDs
					table.insert(alphabet[j].mountIDs, i)
						
					DetectUsableMounts()
					print(mountName.." was added to your Titan Panel [Mount] collection")
			
					return
				end
			end
		end
	end
end

local GetCurrentMount, GetMountInfo
do
    -- Helper functions for handling the information about mounts

    -- Return the name and icon of the currently summoned mount, in that order
    -- Returns nil for each if the player seems not to be mounted
    -- Also sets the mountNumber and mountID 
    function GetCurrentMount()
        local summoned, name, icon, creatureID
        for i = 1, C_MountJournal.GetNumMounts() do
            name, creatureID, icon, active, _, _, _, _, _, _, _  = C_MountJournal.GetMountInfo(i)
            if active then
                mountNumber = i
				--So this is now the creatureID instead of a "spell"
                mountID = creatureID
                return name, icon
            end
        end
        return nil, nil
    end

    -- Returns the information for the current mountNumber and mountSpell
    -- Note that mountNumber is just a cache that usually prevents us from going through all the companions
    -- If mountNumber and mountSpell don't exist, mountNumber is updated to reflect the companion
    -- referenced in mountSpell, unless mountSpell is nil in which case that is updated.
    function GetMountInfo()
        local name, icon, creatureID, active
		
        if mountNumber == nil then
            return nil, nil
        end
		
        name, creatureID, icon, active, _, _, _, _, _, _, _  = C_MountJournal.GetMountInfo(mountNumber)
		
        if creatureID ~= mountID then
            if mountID == nil then
                mountID = creatureID
            else
                for i = 1,  C_MountJournal.GetNumMounts() do
                    name, creatureID, icon, active, _, _, _, _, _, _, _  = C_MountJournal.GetMountInfo(i)
                    if creatureID == mountID then
                        mountNumber = i
                        break
                    end
                end
            end
        end
		
        return name, icon
    end
end

--Sets the text of the titan button
function TitanPanelMountButton_GetButtonText(event)
    local Button = TitanUtils_GetButton(MountID)
    local mountName, mountIcon, mountColor    
    -- No mounts found
    if C_MountJournal.GetNumMounts() == 0 then mountName = "No Mounts Found"
                               mountIcon = "Interface\\Icons\\INV_Misc_QuestionMark"
      -- Not mounted
      elseif IsMounted() == false then mountName = "None"
                                   mountIcon = "Interface\\minimap\\TRACKING\\StableMaster"
        -- On Taxi
        elseif UnitOnTaxi("player") then mountName = "On Taxi"
                                         mountIcon = "Interface\\CURSOR\\taxi"
            -- Mounted, but not cached
            else mountName, mountIcon = GetCurrentMount()
    end
    if IsOutdoors() == nil then
        mountColor = MountC5
    else
        mountColor = MountC3
    end
    Button.registry.icon = mountIcon
    return "Mount: "..mountColor..mountName
end

--Sets the tooltip text shown when mousing over the titan button
function TitanPanelMountButton_GetTooltipText()    
        if IsMounted() then
            if mountNumber == nil then
                GetCurrentMount()
            end
            return MountC3.."Right-click for Mount Menu\n"..MountC3.."Double Left-click to Dismount"
        else
            if mountNumber == nil then
                return MountC3.."Right-click for Mount Menu\n"..MountC3.."Double Left-click to Recall Last Mount"
            else
                local mountName
                mountName = GetMountInfo()
                return MountC3.."Right-click for Mount Menu\n"..MountC3.."Double Left-click to Recall "..MountC4..mountName
        end
    end
end

local PrepareMountSubMenu, ShowFavorite, ShowMenuButton
do
    -- Helper functions for building the menu

    -- Build the submenu for a selection of mounts
    -- Show only mounts belonging to the titleLetter passed
    function PrepareMountSubMenu(titleLetter)
        local MountTestName, owned, icon
        local StartChar
        local info = {}

		--Loop through the alphabet, find the letter which was passed in via the titleLetterParameter
		for i = 1, table.getn(alphabet) do
			if(alphabet[i].letter == titleLetter) then
				--Found the letter, now add that letters mounts...
				for j = 1, table.getn(alphabet[i].mountIDs) do
					MountTestName, _, icon, _, _, _, _, _, _, _, _  = C_MountJournal.GetMountInfo(alphabet[i].mountIDs[j])
					info.text = MountTestName
					info.icon = icon
					info.notCheckable = 1
					info.func = function() TitanPanelMountButton_MountToggle(alphabet[i].mountIDs[j]) end
					UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)  
				end
				
				break
			end
		end
    end
	
	-- Returns the correct index and spell number for a favorite
    function SaveFavoriteMounts()
		local favSelectID1, favSelectID2, favSelectID3
		
		local function GetMountInfoFromText(mountNamePassed)
			for i = 1, C_MountJournal.GetNumMounts() do
				
				local mountName
				--Get dat mount info we need...
				mountName, _, _, _, _, _, _, _, _, _, _  = C_MountJournal.GetMountInfo(i)
				--If the account owns the mount and it starts with the current letter we're on, set found to true for the letter
				if mountNamePassed == mountName then
				
					return i
				end
			end
		end
		
		favSelectID1 = GetMountInfoFromText(favSelection1)
		favSelectID2 = GetMountInfoFromText(favSelection2)
		favSelectID3 = GetMountInfoFromText(favSelection3)
		
		--Make sure it isn't the default text
		if favSelectID1 ~= nil then
			TitanMountFav1 = favSelectID1
		end
		
		if favSelectID2 ~= nil then
		  TitanMountFav2 = favSelectID2
		end
		
		if favSelectID3 ~= nil then
		  TitanMountFav3 = favSelectID3
		end
    end
	
	function PrepareMountFavoriteSubMenu(titleLetter, favoriteDropDown)
        local MountTestName, owned, icon
        local StartChar
        local info = {}

		local function FavoriteMountSelect(self, arg1)
			UIDropDownMenu_SetSelectedID(favoriteDropDown, self:GetID())
			local dropDownName = favoriteDropDown:GetName()
			
			if dropDownName == "DropDownFav1" then
				favSelection1 = arg1
			elseif dropDownName == "DropDownFav2" then
				favSelection2 = arg1
			elseif dropDownName == "DropDownFav3" then
				favSelection3 = arg1
			end
		end
		
		--Loop through the alphabet, find the letter which was passed in via the titleLetterParameter
		for i = 1, table.getn(alphabet) do
			if(alphabet[i].letter == titleLetter) then
				--Found the letter, now add that letters mounts...
				for j = 1, table.getn(alphabet[i].mountIDs) do
					MountTestName, _, icon, _, _, _, _, _, _, _, _  = C_MountJournal.GetMountInfo(alphabet[i].mountIDs[j])
					info.text = MountTestName
					info.icon = icon
					info.func = FavoriteMountSelect
					info.arg1 = MountTestName
					UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)  
				end
				
				break
			end
		end
    end
       
    -- Shows a single favorite line in the main drop down
    -- favorite is the index of the favorite mount, number the number of the favorite (1 for the first, 2 for the second, etc)
    -- modifier is the modifier key for setting that favorite
    function ShowFavorite(favorite, number)
        local info = {}
        if favorite == nil then
            info.text = MountC3.."Favorite #"..number
            info.notCheckable = 0
            --info.func = function() UIErrorsFrame:AddMessage("To set, hold down "..modifier.." and select mount.") end
			info.func = function() ShowFavoriteAddMenu() end
        else
            local name, checked, icon
            name, _, icon, _, _, _, _, _, _, _, _  = C_MountJournal.GetMountInfo(favorite)
            info.text = MountC3..name
			info.icon = icon
			info.notCheckable = 1
            info.func = function() TitanPanelMountButton_MountToggle(favorite) end
        end
		
        UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
    end

	function ShowFavoriteAddMenu()
		local dropdown1 = CreateFrame("Frame", "DropDownFav1", TitanMountFavoriteDialog, "UIDropDownMenuTemplate")
		dropdown1:SetPoint("Center", 50, 50)
		UIDropDownMenu_SetWidth(dropdown1, 200)
		UIDropDownMenu_SetText(dropdown1, "Select Favorite Mount #1")
		
		local dropdown2 = CreateFrame("Frame", "DropDownFav2", TitanMountFavoriteDialog, "UIDropDownMenuTemplate")
		dropdown2:SetPoint("Center", 50, 10)
		UIDropDownMenu_SetWidth(dropdown2, 200)
		UIDropDownMenu_SetText(dropdown2, "Select Favorite Mount #2")

		
		local dropdown3 = CreateFrame("Frame", "DropDownFav3", TitanMountFavoriteDialog, "UIDropDownMenuTemplate")
		dropdown3:SetPoint("Center", 50, -30)
		UIDropDownMenu_SetWidth(dropdown3, 200)
		UIDropDownMenu_SetText(dropdown3, "Select Favorite Mount #3")

		local function initialize(self, level)
			if level == 1 then
				for i = 1, table.getn(alphabet) do
					if alphabet[i].found then
						ShowMenuButton(alphabet[i].letter)
					end
				end
			elseif level == 2 then
				for i = 1, table.getn(alphabet) do
					--Match the MENU_VALUE to the letter in the alphabet
					if UIDROPDOWNMENU_MENU_VALUE == alphabet[i].letter then
						--Prepare the mounts!
						PrepareMountFavoriteSubMenu(alphabet[i].letter, self)
					end
				end
			end
		end
		
		UIDropDownMenu_Initialize(DropDownFav1, initialize)
		UIDropDownMenu_Initialize(DropDownFav2, initialize)
		UIDropDownMenu_Initialize(DropDownFav3, initialize)
		
		if TitanMountFav1 ~= nil then
			local nameText
			name, _, _, _, _, _, _, _, _, _, _  = C_MountJournal.GetMountInfo(TitanMountFav1)
			UIDropDownMenu_SetText(dropdown1, name)
		end
		
		if TitanMountFav2 ~= nil then
			local nameText
			name, _, _, _, _, _, _, _, _, _, _  = C_MountJournal.GetMountInfo(TitanMountFav2)
			UIDropDownMenu_SetText(dropdown2, name)
		end
		
		if TitanMountFav3 ~= nil then
			local nameText
			name, _, _, _, _, _, _, _, _, _, _  = C_MountJournal.GetMountInfo(TitanMountFav3)
			UIDropDownMenu_SetText(dropdown3, name)
		end
		
		TitanMountFavoriteDialog:Show();
	end
	
    -- Creates the menu button for a submenu with title passed
    function ShowMenuButton(title)
        local info = {}
        info.text = title
        info.func = nil
        info.value = title
        info.notCheckable = 1
        info.hasArrow = 1
        UIDropDownMenu_AddButton(info, 1)
    end
end

--Controls how the level 1 and level 2 right click menu is displayed. Each "Arrow Over" is another menu level. 
function TitanPanelRightClickMenu_PrepareMountMenu()
		
        if UIDROPDOWNMENU_MENU_LEVEL == 2 then
            -- We're in a submenu, build it
			for i = 1, table.getn(alphabet) do
				--Match the MENU_VALUE to the letter in the alphabet
				if UIDROPDOWNMENU_MENU_VALUE == alphabet[i].letter then
					--Prepare the mounts!
					PrepareMountSubMenu(alphabet[i].letter)
				end
			end
        else
            -- Build the main menu
            TitanPanelRightClickMenu_AddTitle("Mounts")    
            TitanPanelRightClickMenu_AddSpacer()
        
            ShowFavorite(TitanMountFav1, 1)
            ShowFavorite(TitanMountFav2, 2)
            ShowFavorite(TitanMountFav3, 3)
           
            TitanPanelRightClickMenu_AddSpacer()
            
			--Ok, now iterate through the alphabet again, adding only the letters in which we've found existing, usable mounts
			for i = 1, table.getn(alphabet) do
				if alphabet[i].found then
					ShowMenuButton(alphabet[i].letter)
				end
			end

            TitanPanelRightClickMenu_AddSpacer()
        
            -- The Favorite Dialog button
            if TitanMountFav3 ~= nil or TitanMountFav2 ~= nil or TitanMountFav1 ~= nil then
                local info = {}       
                info.text = "Open Favorite Menu"
                info.value = nil
				info.notCheckable = 1
                info.func = function() ShowFavoriteAddMenu() end
				--function() TitanMountFav3 = nil; TitanMountFav2 = nil; TitanMountFav1 = nil; UIErrorsFrame:AddMessage(MountC3.."Titan Panel [Mount] Favorites Cleared") end
                UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
            end
          
            TitanPanelRightClickMenu_AddToggleIcon(MountID)
		end   
end
 
function TitanPanelMountButton_MountToggle(i)
    local name
    if (not IsModifierKeyDown()) then
        C_MountJournal.Summon(i)
        TitanUtils_CloseRightClickMenu()
        --_, _, mountSpell, _, _ = C_MountJournal.GetMountInfo(i)
        mountNumber = i
    end
end

--Event fired when the titan button is double clicked
function TitanPanelMountButton_OnDoubleClick(button)
    if IsMounted() then
        Dismount()
    elseif button == "LeftButton" and mountNumber ~= nil then
        -- Make sure mountNumber is in sync with mountSpell
        GetMountInfo()
        C_MountJournal.Summon(mountNumber)
    end
end

function TitanPanelMountButton_OnEvent(self, event, arg1, ...)

	if event =="CHAT_MSG_SYSTEM" then
		if strsub(arg1, 1, 25) == "You have added the mount " then
			local mounText
			mountText = string.gsub(arg1, "You have added the mount ", "")
			mountText = string.gsub(mountText, " to your collection.", "")
			
			--Now we have the name of the mount. Look er' up
			AddNewMountID(mountText)
			
			return;
		end
	end
	
    if event == "ADDON_LOADED" then
        mountNumber = mountNumberOver
        mountSpell = mountSpellOver
		return;
    end  
	
    if event == "PLAYER_LOGOUT" then
        mountNumberOver = mountNumber
        mountSpellOver = mountSpell
		return;
    end
	
	if event == "PLAYER_ENTERING_WORLD" then
		DetectUsableMounts()
		--AddNewMountID()
		return;
    end
end

function TitanPanelMountButton_OnUpdate(self)         
    TitanPanelButton_UpdateButton(MountID)
end
